<?php

namespace Modules\Auth\Http\Controllers;

use App\Entities\Eloquent\SfGroups;
use App\Entities\Eloquent\SfGuardUserPermission;
use App\Entities\Eloquent\SfPermissions;
use App\Entities\Eloquent\SfUsers;
use App\Exceptions\UnauthorizedException;
use App\Extensions\Auth\SfGuardUser;
use App\Helper\Helper;
use App\Models\Establecimiento;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Modules\Auth\Dto\V1\TokenResponseDto;
use Modules\Auth\Dto\V1\UserDto;
use Modules\Auth\Http\Requests\V1\LoginRequest;
use Modules\Auth\Service\V1\UsuariosService;

class AuthController extends Controller
{
    /**
     * @var UsuariosService
     */
    private UsuariosService $usuariosService;

    public function __construct(UsuariosService $usuariosService)
    {
        $this->usuariosService = $usuariosService;

    }

    /**
     * Hace la validacion de login.
     * @param LoginRequest
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\Routing\ResponseFactory|\Illuminate\Foundation\Application|\Illuminate\Http\Response
     */
    public function authenticate( LoginRequest $request)
    {
        $username = $request->get('username');
        $password = $request->get('password');

        if(Auth::guard(SfGuardUser::SF_GUARD_USER)->attempt(['username' => $username, 'password' => $password]))
        {

            $user = Auth::guard(SfGuardUser::SF_GUARD_USER)->user();
            $tokens = $user->createToken(config('auth.cookie'))->plainTextToken;

            $respuesta = new TokenResponseDto();

            $respuesta->setTokenType("Bearer");
            $respuesta->setAccessToken($tokens);
		        $user->permissions = SfPermissions::select(array('name','id'))->whereRaw('id in (select permission_id from sf_guard_user_permission where user_id='. $user->id.")")->get();
		        $user->groups = SfGroups::select(array('name','id'))->whereRaw('id in (select group_id from sf_guard_user_group where user_id='. $user->id.")")->get();
		        $user->establecimientos = Establecimiento::select(array('c011_id','c011_nom','c011_aniosecdelfuturo'))->whereRaw('c011_id in (select c011_id from usuario_establecimientos where usuario_id='. $user->id.")")->get();
		        $user->establecimiento_seleccionado =  Establecimiento::select(array('c011_id','c011_nom','c011_aniosecdelfuturo'))->whereRaw('c011_id in (select c011_id from usuario_establecimientos where usuario_id='. $user->id.")")->first();

            return response([
                'user' => $user,
                'token' => $respuesta->getAccessToken()
            ]);
        }
        return response([
            'error' => 'Usuario o contraseña equivocadas'
        ],422);
//        throw new UnauthorizedException(Helper::UNAUTHENTICATED_MESSAGE);
    }

    public function verifyUserToken(Request $request)
    {
        $token = $request->route('token');
        try {

            $user_token = $this->usuariosService->getUserByToken($token);
            if ($user_token)
            {
                if(Auth::guard(SfGuardUser::SF_GUARD_USER)->attemptByUser(['id' => $user_token->id]))
                {
                    $user_auth = Auth::guard(SfGuardUser::SF_GUARD_USER)->user();
                    $tokens = $user_auth->createToken(config('auth.cookie'))->plainTextToken;

                    $respuesta = new TokenResponseDto();
                    $respuesta->setTokenType("Bearer");
                    $respuesta->setAccessToken($tokens);

                    return response([
                        'user' => $user_auth,
                        'token' => $respuesta->getAccessToken()
                    ]);
                }
            }else{

                return response([
                    'error' => 'Usuario o contraseña equivocadas'
                ],422);
            }

        }catch (\Exception $e){
            dd($e->getMessage());
        }

    }


    /**
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\Routing\ResponseFactory|\Illuminate\Foundation\Application|\Illuminate\Http\Response|void
     */
    public function getTokenBySalt(Request $request)
    {
//        $request->validate([
//            'salt'=> 'required | string'
//        ]);
        $salt = $request->route('salt');
        try {

            $user_sf = $this->usuariosService->getUserBySalt($salt);
            if ($user_sf)
            {
                if(Auth::guard(SfGuardUser::SF_GUARD_USER)->attemptByUser(['id' => $user_sf->id]))
                {
                    $user_auth = Auth::guard(SfGuardUser::SF_GUARD_USER)->user();
                    $tokens = $user_auth->createToken(config('auth.cookie'))->plainTextToken;

                    $respuesta = new TokenResponseDto();
                    $respuesta->setTokenType("Bearer");
                    $respuesta->setAccessToken($tokens);

                    return response([
                        'user' => $user_auth,
                        'token' => $respuesta->getAccessToken()
                    ]);
                }
            }else{

                return response([
                    'error' => 'Usuario o contraseña equivocadas'
                ],422);
            }

        }catch (\Exception $e){
            dd($e->getMessage());
        }
    }

    /**
     * Show the form for creating a new resource.
     * @return Renderable
     */
    public function create()
    {
        return view('auth::create');
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Renderable
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show($id)
    {
        return view('auth::show');
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function edit($id)
    {
        return view('auth::edit');
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Renderable
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy($id)
    {
        //
    }

    public function logout(Request $request)
    {

        $request->user()->tokens()->delete();

        return response([
            'success' => true
        ]);


    }

    public function user(){
        $user = Auth::id();

        $user = $this->usuariosService->getUserData($user)->toArray();
        return response()->success( new UserDto($user));
    }


}
