<?php

namespace Modules\OrdenProviciones\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\OrdenProviciones\Services\ListadoLacteosService;
use PHPUnit\Exception;

class ListadoLacteosController extends Controller
{
    protected $service;

    public function __construct(ListadoLacteosService $listadoLacteosService)
    {
        $this->service = $listadoLacteosService;
    }


    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index()
    {
        try {
            return response([
                'listado' => $this->service->getListado()
            ]);
        }catch (\Exception $e){
            dd($e->getMessage());
        }

    }

    public function getListadoByEstablecimiento(Request $request)
    {
        try {
            $establecimiento_id =  $request->route('id');

            return response([
                'listado' => $this->service->getListadoHistorico($establecimiento_id)
            ]);
        }catch (Exception $exception){
            return response('Se produjo un error al consultar el listado',404);
        }
    }

    /**
     * @param Request $request
     */
    public function servicios(Request $request)
    {
        try {
            $establecimiento_id = $request->route('id');
            return response($this->service->getServicios($establecimiento_id));
        }catch (\Exception $e){
            return response('Se produjo un error al intentar obtener los servicios del establecimiento',404);
        }
    }
    /**
     * Show the form for creating a new resource.
     * @return Renderable
     */
    public function create()
    {
        return view('ordenproviciones::create');
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Renderable
     */
    public function store(Request $request)
    {
        try {
            return response($this->service->store($request->request->get('data')));
        }catch (\Exception $exception){
            return response('Se produjo un error al intentar guardar los datos',404);
        }
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show($id)
    {
        return view('ordenproviciones::show');
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function edit($id)
    {
        return view('ordenproviciones::edit');
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Renderable
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy($id)
    {
        //
    }
}
