<?php

namespace Modules\OrdenProviciones\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Client\Response;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;

use Modules\OrdenProviciones\Services\ListadoSolidosService;
use PHPUnit\Exception;

class OrdenProvicionesController extends Controller
{
    protected $listado_service;
    public function __construct(ListadoSolidosService $listadoSolidosService)
    {
        $this->listado_service = $listadoSolidosService;
    }

    public function getListadoSolidos()
    {
        try {

            return response([
                'listado' => $this->listado_service->getListado()
            ]);
        }catch (Exception $exception){
            return response('Se produjo un error al consultar el listado',404);
        }
    }

    public function getListadoByEstablecimiento(Request $request)
    {
        try {
            $establecimiento_id =  $request->route('id');

            return response([
                'listado' => $this->listado_service->getListadoHistorico($establecimiento_id)
            ]);
        }catch (Exception $exception){
            return response('Se produjo un error al consultar el listado',404);
        }
    }

    /**
     * @param Request $request
     * @return Response
     */
    public function getDatosById(Request $request)
    {
        try {
            $establecimiento_id = $request->route('id');

            return response($this->listado_service->getDatosById($establecimiento_id));
        }catch (\Exception $e){
            return response('Se produjo un error al obtener los datos del establecimiento',404);
        }

    }

    /**
     * @param Request $request
     * @return Response
     */
    public function store(Request $request)
    {
        try {
            return response($this->listado_service->store($request->request->get('data')));
        }catch (\Exception $exception){
            return response('Se produjo un error al intentar guardar los datos',404);
        }
    }

    /**
     * @param Request $request
     */
    public function servicios(Request $request)
    {
        try {
            $establecimiento_id = $request->route('id');
            return response($this->listado_service->getServicios($establecimiento_id));
        }catch (\Exception $e){
            return response('Se produjo un error al intentar obtener los servicios del establecimiento',404);
        }
    }

    public function getDesayunoMeriendas(Request $request)
    {
        try {
            $establecimiento_id = $request->route('id');
            $porcentaje = $request->request->get('data');

           $total =  $this->listado_service->getDesayunosMeriendas($establecimiento_id);

           return response(intval($total));
        }catch (\Exception $e){
            return response($e->getMessage(),404);
        }
    }

}
