<?php

namespace Modules\OrdenProviciones\Repositories\V1;

use App\Repository\EloquentRepository;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\DB;
use Modules\OrdenProviciones\Entities\V1\CicloLectivo;
use Modules\OrdenProviciones\Entities\V1\EstablecimientoServicios;

class EstablecimientoServiciosRepository extends EloquentRepository
{
    const LISTADO_SOLIDO_ID = 1;
    const LISTADO_LACTEO_ID = 2;
    public function __construct()
    {
        parent::__construct(new EstablecimientoServicios());
    }

    public function getListadoSolido():Collection | null | string
    {

        $list = DB::table('establecimiento_servicios','s')
            ->select([
                's.id AS id',
                'e.c740_011_id',
                's.establecimiento_id AS establecimiento_id',
                's.nivel_id AS nivel_id',
                'e.c740_codigo AS codigo',
                'e.c740_nombre AS establecimiento',
                'empresas.nombre as empresa',
                'c740_de AS distrito',
                's.cantidad AS cantidad',
                's.dias AS dias',
                's.vigente',
                's.servicio_id AS servicio_id',
                'se.descripcion AS servicio',
                's.observacion AS observacion',
                's.fecha_inicio as fecha_inicio',
                's.fecha_modificacion as fecha_modificacion',
                'cl.c257_ciclolectivo AS ciclo_lectivo'

            ])
            ->join('740_ESTABLECIMIENTOS_BECAS AS e','e.c740_id','=','s.establecimiento_id')
            ->join('establecimiento_empresas','establecimiento_empresas.establecimiento_id','=','s.establecimiento_id')
            ->join('empresas','empresas.id','=','establecimiento_empresas.empresa_id')
            ->join('servicios AS se','se.id','=','s.servicio_id')
            ->join('257_CICLO_LECTIVO AS cl','cl.c257_id','=','s.ciclo_lectivo_id')
            ->where('s.vigente','=',1)
            ->where('s.ciclo_lectivo_id','=',CicloLectivo::max("c257_id"))
            ->where('s.listado_id','=',self::LISTADO_SOLIDO_ID)
            ->where('establecimiento_empresas.listado_id','=',1)
            ->orderBy('e.c740_id')
            ->get()
        ;


        return $list;
    }

    public function getListadoLacteo($vigente):Collection | null | string
    {

        $list = DB::table('establecimiento_servicios','s')
            ->select([
                's.id AS id',
                'e.c740_011_id',
                's.establecimiento_id AS establecimiento_id',
                's.nivel_id AS nivel_id',
                'e.c740_codigo AS codigo',
                'e.c740_nombre AS establecimiento',
                'empresas.nombre as empresa',
                'c740_de AS distrito',
                's.cantidad AS cantidad',
                's.dias AS dias',
                's.vigente',
                's.servicio_id AS servicio_id',
                'se.descripcion AS servicio',
                's.observacion AS observacion',
                's.fecha_inicio as fecha_inicio',
                's.fecha_modificacion as fecha_modificacion',
                'cl.c257_ciclolectivo AS ciclo_lectivo',
                's.ajuste_consumo_yogur',
                's.porcentaje_ajuste_por_consumo_yogur',
                's.ajuste_consumo_leche',
                's.porcentaje_ajuste_leche',
                's.domicilio_entrega'

            ])
            ->join('740_ESTABLECIMIENTOS_BECAS AS e','e.c740_id','=','s.establecimiento_id')
            ->join('establecimiento_empresas','establecimiento_empresas.establecimiento_id','=','s.establecimiento_id')
            ->join('empresas','empresas.id','=','establecimiento_empresas.empresa_id')
            ->join('servicios AS se','se.id','=','s.servicio_id')
            ->join('257_CICLO_LECTIVO AS cl','cl.c257_id','=','s.ciclo_lectivo_id')
            ->where('s.vigente','=',$vigente)
            ->where('s.ciclo_lectivo_id','=',CicloLectivo::max("c257_id"))
            ->where('s.listado_id','=',self::LISTADO_LACTEO_ID)
            ->where('establecimiento_empresas.listado_id','=',2)
            ->orderBy('e.c740_id')
            ->get()
        ;


        return $list;
    }

    public function getListadoLacteoHsitorico($vigente,$establecimiento_id):Collection | null | string
    {

        $list = DB::table('establecimiento_servicios','s')
            ->select([
                's.id AS id',
                'e.c740_011_id',
                's.ref_id',
                's.establecimiento_id AS establecimiento_id',
                's.nivel_id AS nivel_id',
                'e.c740_codigo AS codigo',
                'e.c740_nombre AS establecimiento',
                'empresas.nombre as empresa',
                'c740_de AS distrito',
                's.cantidad AS cantidad',
                's.dias AS dias',
                's.vigente',
                's.servicio_id AS servicio_id',
                'se.descripcion AS servicio',
                's.observacion AS observacion',
                's.fecha_inicio as fecha_inicio',
                's.fecha_modificacion as fecha_modificacion',
                'cl.c257_ciclolectivo AS ciclo_lectivo',
                's.ajuste_consumo_yogur',
                's.porcentaje_ajuste_por_consumo_yogur',
                's.ajuste_consumo_leche',
                's.porcentaje_ajuste_leche',
                's.domicilio_entrega'

            ])
            ->join('740_ESTABLECIMIENTOS_BECAS AS e','e.c740_id','=','s.establecimiento_id')
            ->join('establecimiento_empresas','establecimiento_empresas.establecimiento_id','=','s.establecimiento_id')
            ->join('empresas','empresas.id','=','establecimiento_empresas.empresa_id')
            ->join('servicios AS se','se.id','=','s.servicio_id')
            ->join('257_CICLO_LECTIVO AS cl','cl.c257_id','=','s.ciclo_lectivo_id')
            ->where('s.vigente','=',$vigente)
            ->where('s.ciclo_lectivo_id','=',CicloLectivo::max("c257_id"))
            ->where('s.listado_id','=',self::LISTADO_LACTEO_ID)
            ->where('s.establecimiento_id','=',$establecimiento_id)
            ->orderBy('e.c740_id')
            ->get()
        ;


        return $list;
    }

    public function getServiciosByEstablecimiento($establecimiento_id,$tipo_listado)
    {
        return DB::table('establecimiento_servicios' ,'es')
            ->select([
                'es.id as id',
                'es.establecimiento_id',
                'es.nivel_id',
                'es.observacion',
                'es.dias',
                'es.cantidad',
                's.id as servicio_id',
                's.descripcion as servicio',
                'es.ajuste_consumo_yogur',
                'es.porcentaje_ajuste_por_consumo_yogur',
                'es.ajuste_consumo_leche',
                'es.porcentaje_ajuste_leche',
                'es.domicilio_entrega'
            ])
            ->join('servicios AS s','s.id','=','es.servicio_id')
            ->where('es.establecimiento_id','=',$establecimiento_id)
            ->where('es.vigente','=',1)
            ->where('es.listado_id','=',$tipo_listado)
            ->get();

    }

    public function getHistorico($establecimiento_id)
    {

        $list = DB::table('establecimiento_servicios','s')
            ->select([
                's.id AS id',
                's.ref_id',
                'e.c740_011_id',
                's.establecimiento_id AS establecimiento_id',
                's.nivel_id AS nivel_id',
                'e.c740_codigo AS codigo',
                'e.c740_nombre AS establecimiento',
                'empresas.nombre as empresa',
                'c740_de AS distrito',
                's.cantidad AS cantidad',
                's.dias AS dias',
                's.vigente',
                's.servicio_id AS servicio_id',
                'se.descripcion AS servicio',
                's.observacion AS observacion',
                's.fecha_inicio as fecha_inicio',
                's.fecha_modificacion as fecha_modificacion'

            ])
            ->join('740_ESTABLECIMIENTOS_BECAS AS e','e.c740_id','=','s.establecimiento_id')
            ->join('establecimiento_empresas','establecimiento_empresas.establecimiento_id','=','e.c740_id')
            ->join('empresas','empresas.id','=','establecimiento_empresas.empresa_id')
            ->join('servicios AS se','se.id','=','s.servicio_id')
            ->where('s.vigente','=',0)
            ->where('s.establecimiento_id','=',$establecimiento_id)
            ->where('s.listado_id','=',self::LISTADO_SOLIDO_ID)
            ->orderBy('s.id','desc')
            ->get();

        return $list;
    }

    public function getAllServices($establecimiento_id)
    {
        return $this->model->where('establecimiento_id',$establecimiento_id)->where('vigente',1)->get();
    }

    public function getDatosById($establecimiento_id){
//        dd($establecimiento_id);
        return DB::table('establecimiento_servicios','s')
            ->select([
                's.id AS id',
                's.establecimiento_id AS establecimiento_id',
                's.nivel_id AS nivel_id',
                'e.c740_codigo AS codigo',
                'e.c740_011_id',
                'e.c740_nombre AS establecimiento',
                'empresas.nombre as empresa',
                'c740_de AS distrito',
                's.cantidad AS cantidad',
                's.dias AS dias',
                's.vigente',
                's.servicio_id AS servicio_id',
                'se.descripcion AS servicio',
                's.observacion AS observacion',
                's.fecha_inicio as fecha_inicio',
                's.fecha_modificacion as fecha_modificacion'

            ])
            ->join('740_ESTABLECIMIENTOS_BECAS AS e','e.c740_id','=','s.establecimiento_id')
            ->join('establecimiento_empresas','establecimiento_empresas.establecimiento_id','=','e.c740_id')
            ->join('empresas','empresas.id','=','establecimiento_empresas.empresa_id')
            ->join('servicios AS se','se.id','=','s.servicio_id')
            ->where('s.vigente','=',1)
            ->where('e.c740_id',$establecimiento_id)
            ->orderBy('e.c740_id')
            ->get();
    }

    public function getDesayunosMeriendas($establecimiento_id)
    {
        return DB::table('establecimiento_servicios','es')
                ->where('es.establecimiento_id','=',$establecimiento_id)
                ->where('es.listado_id','=',1)
                ->whereIn('es.servicio_id',[2,3])
                ->where('es.vigente','=',1)
                ->sum('es.cantidad');

    }

}
