<?php

namespace Modules\OrdenProviciones\Services;

use App\Service\BaseService;
use Illuminate\Support\Facades\DB;
use Modules\OrdenProviciones\Dto\ServicioLacteoSaveDto;
use Modules\OrdenProviciones\Dto\ServicioSaveDto;
use Modules\OrdenProviciones\Dto\ServiciosDto;
use Modules\OrdenProviciones\Entities\V1\EstablecimientoServicios;
use Modules\OrdenProviciones\Repositories\V1\EstablecimientoServiciosRepository;

class ListadoLacteosService extends BaseService
{

    protected $helper;
    protected $servico_establecimiento_repo;
    const VIGENTE_ACTIVO = 1;
    const VIGENTE_DESACTIVO = 0;
    const LISTADO_LACTEO_ID = 2;

    public function __construct(HelperService $helperService, EstablecimientoServiciosRepository $establecimientoServiciosRepository)
    {
        $this->helper = $helperService;
        $this->servico_establecimiento_repo = $establecimientoServiciosRepository;
    }

    public function getListado()
    {
        try {
            $coleccionServicios = $this->servico_establecimiento_repo->getListadoLacteo(self::VIGENTE_ACTIVO);

            $coleccionServicios->transform(function ($item) {
                return (array)$item;
            });

            $serviciosPorColegio = $this->helper->serviciosPorColegioLacteos($coleccionServicios);

            return $this->helper->setListadoLacteo($serviciosPorColegio);
        }catch (\Exception $e){
            throw new \Exception('Error al intentar coneguir el litado');
        }

    }

    public function getListadoHistorico($establecimiento_id)
    {
        try {

            $coleccionServicios = $this->servico_establecimiento_repo->getListadoLacteoHsitorico(self::VIGENTE_DESACTIVO,$establecimiento_id);

            $coleccionServicios->transform(function ($item) {
                return (array)$item;
            });

            $serviciosPorColegio = $this->helper->serviciosPorColegioLacteosHistorico($coleccionServicios);

            return $this->helper->setListadoLacteo($serviciosPorColegio);
        }catch (\Exception $e){
            throw new \Exception('Error al intentar coneguir el litado');
        }

    }

    public function getServicios($establecimiento_id)
    {
        $servcios =[];
        $servicios_establecimiento = $this->servico_establecimiento_repo->getServiciosByEstablecimiento($establecimiento_id,self::LISTADO_LACTEO_ID);

        foreach ($servicios_establecimiento as $service)
        {
            $service_arr = json_decode(json_encode($service), true);

            $dto = new ServiciosDto($service_arr);
            $dto->setDias(json_decode($service->dias));
            $servcios[] = $dto;
        }
        return ['servicios' => $servcios];
    }


    public function store($request)
    {
        $save_dto = [];
        $forms = json_decode($request);
        $establecimiento_id = $forms[0]->establecimiento_id;
        $servicios_vigentes = EstablecimientoServicios::where('establecimiento_id',$establecimiento_id)
            ->where('vigente',self::VIGENTE_ACTIVO)
            ->where('listado_id',self::LISTADO_LACTEO_ID)
            ->get();


        foreach ($forms as $service)
        {
            $data = json_decode(json_encode($service), true);

            $save_dto[] = new ServicioLacteoSaveDto($data);
        }

        try {
            DB::beginTransaction();
            $ref_id = $this->helper->getRefId();
            $servicios_vigentes->each(function ($item) use ($ref_id){
                $item->update(['vigente'=> 0,'fecha_modificacion' => new \DateTime(),'ref_id' => $ref_id]);
            });

            foreach ($save_dto as $service)
            {
                $datos = $service->toArray();
                $datos['dias'] = json_encode($service->getDias());
                EstablecimientoServicios::create($datos);
            }
            DB::commit();
            return $save_dto;
        }catch (\Exception $exception){
            DB::rollBack();
            dd($exception->getMessage(),$exception->getFile(),$exception->getLine(),$exception->getTrace());
        }

    }

}
