<?php

namespace Modules\OrdenProviciones\Services;

use App\Service\BaseService;
use Illuminate\Support\Facades\DB;
use Modules\OrdenProviciones\Dto\BecasEstablecimientoDto;
use Modules\OrdenProviciones\Dto\CicloLectivoDto;
use Modules\OrdenProviciones\Dto\DatosEstablecimientoDto;
use Modules\OrdenProviciones\Dto\InfoListadoDto;
use Modules\OrdenProviciones\Dto\ListadoSolidoDto;
use Modules\OrdenProviciones\Dto\MatriculasDto;
use Modules\OrdenProviciones\Dto\ServicioFormDto;
use Modules\OrdenProviciones\Dto\ServicioSaveDto;
use Modules\OrdenProviciones\Dto\ServiciosDto;
use Modules\OrdenProviciones\Entities\V1\EstablecimientoServicios;
use Modules\OrdenProviciones\Repositories\V1\CicloLectivoRepository;
use Modules\OrdenProviciones\Repositories\V1\EstablecimientosBecasRepository;
use Modules\OrdenProviciones\Repositories\V1\EstablecimientoServiciosRepository;
use Modules\OrdenProviciones\Repositories\V1\ListadoSolidosRepository;
use Modules\OrdenProviciones\Repositories\V1\MatriculaEscuelasRepository;

class ListadoSolidosService extends BaseService
{
    const LISTADO_SOLIDO_ID = 1;

    protected $ciclo_lectivo_repo;
    protected $establecimientos_repo;
    protected  $matriculas_repo;
    protected $servico_establecimiento_repo;
    protected  $helper;


    public function __construct( CicloLectivoRepository $cicloLectivoRepository, EstablecimientosBecasRepository $establecimientosBecasRepository, MatriculaEscuelasRepository $matriculaEscuelasRepository,EstablecimientoServiciosRepository $serviciosRepository, HelperService $helperService)
    {

        $this->ciclo_lectivo_repo = $cicloLectivoRepository;
        $this->establecimientos_repo = $establecimientosBecasRepository;
        $this->matriculas_repo = $matriculaEscuelasRepository;
        $this->servico_establecimiento_repo = $serviciosRepository;
        $this->helper = $helperService;
    }

    /**
     * @return array
     * @throws \Exception
     */
    public function getListado()
    {
        try {
            $coleccionServicios = $this->servico_establecimiento_repo->getListadoSolido();
            $coleccionServicios->transform(function ($item) {
                return (array)$item;
            });

            $serviciosPorColegio = $this->helper->serviciosPorColegio($coleccionServicios);

            return $this->helper->setListado($serviciosPorColegio);
        }catch (\Exception $e){
            throw new \Exception('Error al intentar coneguir el litado');
        }

    }

    /**
     * @param $establecimiento_id
     * @return array|mixed|ListadoSolidoDto
     * @throws \Exception
     */
    public function getListadoHistorico($establecimiento_id)
    {
        try {

            $coleccionServicios = $this->servico_establecimiento_repo->getHistorico($establecimiento_id);

            $coleccionServicios->transform(function ($item) {
                return (array)$item;
            });

            $serviciosPorColegio = $this->helper->serviciosPorColegioHistorico($coleccionServicios);

            return $this->helper->setListado($serviciosPorColegio);
        }catch (\Exception $e){
            throw new \Exception('Error al intentar coneguir el litado');
        }

    }

    /**
     * @param $establecimiento_id
     * @return array[]
     */
    public function getDatosById($establecimiento_id)
    {
        $ciclo_lectivo = $this->ciclo_lectivo_repo->getMayorCicloLectivo();

        $ciclo_lectivo->transform(function ($item) {
            return (array)$item;
        });

        $cl = new CicloLectivoDto($ciclo_lectivo[0]);

        $coleccionServicios =  $this->servico_establecimiento_repo->getDatosById($establecimiento_id);

        $coleccionServicios->transform(function ($item) {
            return (array)$item;
        });

        $serviciosPorColegio = $this->helper->serviciosPorColegio($coleccionServicios);

        $listado = $this->helper->setListado($serviciosPorColegio,false);

        if ($listado->getBecaId()){

            $matriculas = $this->matriculas_repo->getMatriculaByEstablecimientoId($listado->getBecaId());

            $becas = $this->establecimientos_repo->getBecasPorEstablecimiento($listado->getBecaId());

            if (count($becas) > 0)
            {
                $becas->transform(function ($item) {
                    return (array)$item;
                });
                $becas_dto = new BecasEstablecimientoDto($becas[0]);
            }else{
                $becas_dto = new BecasEstablecimientoDto();
            }


            if (!is_null($matriculas[0]->matricula))
            {
                $matriculas->transform(function ($item){
                    return (array)$item;
                });

                $matricula_dto = new MatriculasDto($matriculas[0]);
                $matricula_dto->setApertura($matriculas[0]['datos']);
            }else{
                $matricula_dto = new MatriculasDto();
            }

        }

        $datos_establecimiento = new DatosEstablecimientoDto();
        $datos_establecimiento->setBecasEstablecimiento($becas_dto);
        $datos_establecimiento->setListadoSolido($listado);
        $datos_establecimiento->setCicloLectivo($cl);
        $datos_establecimiento->setMatricula($matricula_dto);

        $aperturas = [];

        if ($datos_establecimiento->getMatricula()->getApertura())
        {
            foreach ($datos_establecimiento->getMatricula()->getApertura() as $apertura)
            {
                $aperturas[] = $apertura->toArray();
            }
        }



        return [
            'datos' => [
                'ciclo_lectivo' => $datos_establecimiento->getCicloLectivo()->toArray(),
                'becas_establecimiento' => $datos_establecimiento->getBecasEstablecimiento()->toArray(),
                'listado_solido' => $datos_establecimiento->getListadoSolido()->toArray(),
                'matricula' => $datos_establecimiento->getMatricula()->toArray(),
                'apertura' => $aperturas,
                'servicios' => $datos_establecimiento->getListadoSolido()->getInfo(),
                'formServicios' => $this->helper->setFormServicios($datos_establecimiento->getListadoSolido()->getInfo(), $establecimiento_id)
                    ]
            ];

    }


    /**
     * @param $request
     * @return array|void
     */
    public function store($request)
    {
        $save_dto = [];
        $forms = json_decode($request);

        $establecimiento_id = $forms[0]->establecimiento_id;
        $servicios_vigentes = EstablecimientoServicios::where('establecimiento_id',$establecimiento_id)
            ->where('vigente',1)
            ->where('listado_id',self::LISTADO_SOLIDO_ID)
            ->get();


        foreach ($forms as $service)
        {
            $data = json_decode(json_encode($service), true);
            $save_dto[] = new ServicioSaveDto($data);
        }

        try {
            DB::beginTransaction();
            $ref_id = $this->helper->getRefId();
            $servicios_vigentes->each(function ($item) use ($ref_id){
                $item->update(['vigente'=> 0,'fecha_modificacion' => new \DateTime(),'ref_id' => $ref_id]);
            });

            foreach ($save_dto as $service)
            {
                $datos = $service->toArray();
                $datos['dias'] = json_encode($service->getDias());
                EstablecimientoServicios::create($datos);
            }
            DB::commit();
            return $save_dto;
        }catch (\Exception $exception){
            DB::rollBack();
            dd($exception->getMessage(),$exception->getFile(),$exception->getLine(),$exception->getTrace());
        }

    }

    /**
     * @param $establecimiento_id
     * @return array[]
     */
    public function getServicios($establecimiento_id)
    {
        $servcios =[];
        $servicios_establecimiento = $this->servico_establecimiento_repo->getServiciosByEstablecimiento($establecimiento_id,self::LISTADO_SOLIDO_ID);

        foreach ($servicios_establecimiento as $service){
            $dto = new ServiciosDto();
            $dto->setId($service->id);
            $dto->setEstablecimientoId($service->establecimiento_id);
            $dto->setNivelId($service->nivel_id);
            $dto->setObservacion($service->observacion);
            $dto->setDias(json_decode($service->dias));
            $dto->setServicio($service->servicio);
            $dto->setServicioId($service->servicio_id);
            $dto->setCantidad($service->cantidad);

            $servcios[] = $dto;
        }
        return ['servicios' => $servcios];
    }

    public function getDesayunosMeriendas($establecimiento_id)
    {
        try {
            return $this->servico_establecimiento_repo->getDesayunosMeriendas($establecimiento_id);
        }catch (\Exception $e){
            throw new \Exception('error al conseguir los datos');
        }
    }

}
