<?php



namespace App\Repository;


use Illuminate\Database\Eloquent\Model;

/**
 * Class EloquentRepository
 * @package App\Repository
 */
class EloquentRepository implements RepositoryInterface
{
    /**
     * @var
     */
    protected $model;

    /**
     * @param Model $model
     *
     */
    public function __construct(Model $model) {
        $this->model = $model;
    }

    /**
     * @return Model
     */
    public function getModel() {
        return $this->model;
    }

    /**
     * @param $model
     * @return $this
     */
    public function setModel($model) {
        $this->model = $model;
        return $this;
    }

    /**
     * @param array $columns
     * @return mixed
     */
    public function all($columns = array('*')) {
        return $this->model->get($columns);
	}

    public function allWithTrashed($columns = array('*')) {
        return $this->model->withTrashed()->get($columns);
	}

	/**
     * @param array $filtro
     * @param array $columns
     * @return mixed
     */
	public function getAllBy(array $filtro = [], $columns = array('*')) {
		return $this->model->where($filtro)->get($columns);
	}

    /**
     * @param int $perPage
     * @param array $columns
     * @return mixed
     */
    public function paginate($perPage = 15, $columns = array('*')) {
        return $this->model->paginate($perPage, $columns);
    }

    /**
     * @param array $data
     * @return mixed
     */
    public function create(array $data) {
        return $this->model->create($data);
    }

    /**
     * @param array $data
     * @param $id
     * @param string $attribute
     * @return mixed
     */
    public function update(array $data, $id, $attribute="id") {
    	return $this->model->where($attribute, '=', $id)->update($data);
    }

    /**
     * @param $id
     * @return mixed
     */
    public function delete($id) {
        return $this->model->destroy($id);
    }

    /**
     * @param $id
     * @param array $columns
     * @return mixed
     */
    public function find($id, $columns = array('*')) {
        return $this->model->find($id, $columns);
    }

    public function findWithTrashed($id, $columns = array('*')) {
        return $this->model->withTrashed()->find($id, $columns);
    }

    /**
     * @param $attribute
     * @param $value
     * @param array $columns
     * @return mixed
     */
    public function findBy($attribute, $value, $columns = array('*')) {
        return $this->model->where($attribute, '=', $value)->first($columns);
	}

    public function findByFilters(array $filtro = [] , $columns = array('*')) {
        return $this->model->where($filtro)->first($columns);
	}


    public function findByWithTrashed($attribute, $value, $columns = array('*')) {
        return $this->model->withTrashed()->where($attribute, '=', $value)->first($columns);
	}

    /**
     * @param $attribute
     * @param $value
     * @param array $columns
     * @return mixed
     */
    public function findAllBy($attribute, $value, $columns = array('*')) {
        return $this->model->where($attribute, '=', $value)->get($columns);
    }
}
