<?php

namespace App\Service\Search;
use http\Env\Request;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Validation\ValidationException;

abstract class AbstractSearchService
{
    protected $request;

    protected $searchable = [];

    protected $exclude_columns = ['page', 'per_page', 'sort_by', 'sort_order'];

    private $search_keys = [
        'lt', 'gt', 'bt', 'ne', 'lk', 'begin', 'end', 'notLk', 'notBegin', 'notEnd', 'in','notIn', 'null', 'notNull'
    ];

    protected $order_by_related_tables = [];

    protected $scope_key = 'scope_';

    protected $map_columns = [];

    protected $clean_values = [];


    protected function find($query){

        $request = $this->request->except($this->exclude_columns);

        foreach($request as $key => $value) {
            if(empty($value))
                continue;

            $column = ($this->map_columns[$key])??$key;
            $q = $query;

            if($key == 'order_by' && trim($value)!=''){
                $this->addOrder($q, $value);
                continue;
            }

            if(count($this->searchable)>0 && !in_array($key,$this->searchable ))
                continue;

            $is_related = strpos($column, '.');

            if( $is_related !== false ){
                list($relation, $column_name)= explode(".", $column);
                $q->with($relation)->whereHas($relation, function($q_related) use ($column_name, $value){
                    $this->addCondition($q_related, $column_name, $value);
                });
            }else{
                $this->addCondition($q, $column, $value);
            }
        }
       return $query;

    }

    private function  addCondition($q, $column, $value)
    {
        if ($this->createScopeClause( $q, $value, $column)) {
            return true;
        }
        if ($this->createSearchClause( $q, $value, $column)) {
            return true;
        }
        $q->where($column, '=', $value);
        return true;

    }

    protected function addOrder($q,  $value)
    {

        $orders = explode(',',$value);

        $related_order = array();

        foreach($orders as $order){
            $order_by = explode(' ', trim($order));
            $order_dir =$order_by[1]??'asc';
            $order_field = $order_by[0];
            $is_order_related = strpos($order_field, '.');

            if( $is_order_related === false ){
                $q->orderBy($order_field, $order_dir);
            }else{

                list($relation, $column_name)= explode(".", $order_field);

                $join_data = $this->order_by_related_tables[strtolower($relation)]??false;

                if(is_array($join_data)){
                    list($table, $column1, $column2)= $join_data;
                    $related_order[$relation] = array($table, $column1, $column2,$column_name,$order_dir);

                }else{
                    throw ValidationException::withMessages([$column_name => 'El orden no es valido']);
                }
            }

        }
        foreach ($related_order as $order){
            list($table, $column1, $column2,$column_name,$order_dir)= $order;
            $q->join($table, $column1, '=',$column2)->orderBy("$table.$column_name",$order_dir);
        }
        return true;
    }

    protected function createScopeClause($query, $value,  $column): bool
    {

        if (substr($column, 0, 6) === $this->scope_key) {
            list(, $scope)= explode("_", $column);
            $query->{$scope}($value);
            return true;
        }

        return false;

    }


    protected function createSearchClause($query, $value,  $column): bool
    {
        $pos = strpos($value, ':');
        if ($pos === false) {
            return false;
        }
        foreach ($this->search_keys as $type) {
            if (substr($value, 0, (strlen($type) + 1 )) === $type.':') {

                $value = trim(substr($value, strlen($type) + 1));

                if(in_array($type, ['in','notIn','bt'])){
                    $value = !(is_array($value))?explode(',',$value):$value;
                }

                switch ($type) {
                    case 'gt':
                        $query->where($column, '>=', $value, 'and');
                        break;
                    case 'lt':
                        $query->where($column, '<=', $value, 'and');
                        break;
                    case 'ne':
                        $query->where($column, '<>', $value, 'and');
                        break;
                    case 'bt':
                        $query->whereBetween($column, $value);
                        break;
                    case 'in':
                        $query->whereIn($column, $value);
                        break;
                    case 'notIn':
                        $query->whereNotIn($column, $value);
                        break;
                    case 'lk':
                        $query->where($column, 'LIKE', '%'.$value.'%');
                        break;
                    case 'begin':
                        $query->where($column, 'LIKE', $value.'%');
                        break;
                    case 'end':
                        $query->where($column, 'LIKE', '%'.$value);
                        break;
                    case 'notLk':
                        $query->where($column, 'NOT LIKE', '%'.$value.'%');
                        break;
                    case 'notBegin':
                        $query->where($column, 'NOT LIKE', $value.'%');
                        break;
                    case 'notEnd':
                        $query->where($column, 'NOT LIKE', '%'.$value);
                        break;
                    case 'null':
                        $query->whereNull($column);
                        break;
                    case 'notNull':
                        $query->whereNotNull($column);
                        break;
                }
                return true;
            }
        }

        return false;
    }




}
